@@ -17,13 +17,13 @@
 %                            September 1994                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -1485,7 +1485,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   if (elapsed_time > MagickEpsilon)
     {
       (void) FormatMagickSize((MagickSizeType) ((double) image->columns*
-        image->rows/elapsed_time+0.5),MagickFalse,"B",MagickPathExtent,format);
+        image->rows/elapsed_time+0.5),MagickFalse,"P",MagickPathExtent,format);
       (void) FormatLocaleFile(file,"  Pixels per second: %s\n",format);
     }
   (void) FormatLocaleFile(file,"  User time: %0.3fu\n",user_time);
