@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
   
-    https://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -27,7 +27,8 @@ extern MagickPrivate MagickBooleanType
   MagickComponentGenesis(void);
 
 extern MagickPrivate void
-  MagickComponentTerminus(void);
+  MagickComponentTerminus(void),
+  ResetMagickPrecision(void);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
