@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
 
-    https://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -29,7 +29,7 @@ extern "C" {
 #define MaxMultibyteCodes  6
 
 extern MagickPrivate MagickBooleanType
-  IsGlob(const char *);
+  IsGlob(const char *) magick_attribute((__pure__));
 
 typedef struct
 {
